#include <stdio.h>
#include <string.h>
#include "nrf_drv_timer.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_pwm.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "boards.h"
#include "bsp.h"
#include "nrf_drv_clock.h"
#include "nrf_delay.h"
#include "nrf_drv_gpiote.h"
#include "nrfx_timer.h"
#include "nrf_pwr_mgmt.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"


#define OUTPUT_PIN LED_1
#define OUTPUT_PIN2 LED_2
#define PIN_OUT LED_3

#define    PIN_IN    22
//#define    PWM_PIN   11

unsigned int PWM_DELAY_US = 1000 ;


unsigned int duty = 10000 ;
static int  gpio_flag;
uint8_t check;
  


static nrf_drv_pwm_t m_pwm0 = NRF_DRV_PWM_INSTANCE(0);
//static nrf_drv_pwm_t m_pwm1 = NRF_DRV_PWM_INSTANCE(2);
const nrfx_timer_t PWM_DELAY_TIMER = NRFX_TIMER_INSTANCE(1);
static nrf_ppi_channel_t ppi_channel_1; // A struct to hold ppi channel values
static nrf_ppi_channel_t ppi_channel_2;

uint32_t compare_event_addr;
uint32_t pwm_task_addr;
uint32_t gpiote_task_addr;
uint32_t timer_ticks;


// Declare variables holding PWM sequence values. In this example only one channel is used 
nrf_pwm_values_individual_t seq_values[] = {10000,10000 | 0x8000};
//nrf_pwm_values_individual_t seq1_values[] = {10000};

nrf_pwm_sequence_t const seq =
{
    .values.p_individual = seq_values,
    .length          = NRF_PWM_VALUES_LENGTH(seq_values),
    .repeats         = 0,
    .end_delay       = 0
};

/*
nrf_pwm_sequence_t const seq1 =
{
    .values.p_individual = seq1_values,
    .length          = NRF_PWM_VALUES_LENGTH(seq1_values),
    .repeats         = 0,
    .end_delay       = 0
};
*/
void pwm_delay_timeout_handler(nrf_timer_event_t event_type, void* p_context)
{
    
       
    
}

static void disable_pwm_startup_peripherals()
{
 nrf_drv_gpiote_in_event_disable(PIN_IN);
 nrf_drv_ppi_channel_disable(ppi_channel_1);
 nrfx_timer_disable(&PWM_DELAY_TIMER);
 nrf_drv_ppi_channel_disable(ppi_channel_2);

}

static void irq_handler(nrfx_pwm_evt_type_t event_type)
{
 
 switch(event_type)
 {
   case NRFX_PWM_EVT_FINISHED:
        NRF_LOG_INFO("PWM IS STARTED");
        disable_pwm_startup_peripherals();
        break;
   default:
   // nothing
        break;
 }
}


void in_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
   
  //nrf_drv_gpiote_in_event_disable(PIN_IN);
}

static void ppi_init(void)
{
    uint32_t err_code = NRF_SUCCESS;

    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);
    
    

    // Allocate the channel from the available PPI channels
    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel_1);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_ppi_channel_assign(ppi_channel_1,
                                          nrf_drv_gpiote_in_event_addr_get(PIN_IN),
                                          nrf_drv_timer_task_address_get(&PWM_DELAY_TIMER,
                                                                         NRF_TIMER_TASK_START));
    APP_ERROR_CHECK(err_code);


    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel_2);
    APP_ERROR_CHECK(err_code);

    compare_event_addr = nrf_drv_timer_event_address_get(&PWM_DELAY_TIMER,NRF_TIMER_EVENT_COMPARE0);
    //pwm_task_addr =  nrf_drv_pwm_task_address_get(&m_pwm0,NRF_PWM_TASK_SEQSTART0);
    //gpiote_task_addr = nrf_drv_gpiote_out_task_addr_get(PIN_OUT);

    err_code = nrf_drv_ppi_channel_assign(ppi_channel_2, compare_event_addr,pwm_task_addr);
    APP_ERROR_CHECK(err_code);


    err_code = nrf_drv_ppi_channel_enable(ppi_channel_1);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_ppi_channel_enable(ppi_channel_2);
    APP_ERROR_CHECK(err_code);
 }

static void gpio_init(void)
{
    ret_code_t err_code;
    

    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);

    
    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(true);
    in_config.pull = NRF_GPIO_PIN_PULLDOWN;

    err_code = nrf_drv_gpiote_in_init(PIN_IN, &in_config, in_pin_handler);
    APP_ERROR_CHECK(err_code);

    //nrf_drv_gpiote_out_config_t config = GPIOTE_CONFIG_OUT_TASK_TOGGLE(false);

    //err_code = nrf_drv_gpiote_out_init(PIN_OUT, &config);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_event_enable(PIN_IN, true);
    //nrf_drv_gpiote_out_task_enable(PIN_OUT);
}

void timers_init(void)
{
    uint32_t err_code;
    
    
    
    nrfx_timer_config_t timer_cfg = NRFX_TIMER_DEFAULT_CONFIG;
    timer_cfg.frequency = NRF_TIMER_FREQ_1MHz;
    err_code = nrfx_timer_init(&PWM_DELAY_TIMER, &timer_cfg,pwm_delay_timeout_handler);
    APP_ERROR_CHECK(err_code);

    timer_ticks = nrfx_timer_us_to_ticks(&PWM_DELAY_TIMER, PWM_DELAY_US);

    nrfx_timer_extended_compare(&PWM_DELAY_TIMER, 
                                  NRF_TIMER_CC_CHANNEL0, 
                                  timer_ticks,
                                  NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK,
                                  true);

   nrfx_timer_enable(&PWM_DELAY_TIMER);

}

/*
static void pwm1_init(void)
{
    nrf_drv_pwm_config_t const config0 =
    {
        .output_pins =
        {
            PWM_PIN, // channel 0
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 1
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 2
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 3
        },
        .irq_priority = APP_IRQ_PRIORITY_LOWEST,
        .base_clock   = NRF_PWM_CLK_1MHz,
        .count_mode   = NRF_PWM_MODE_UP,
        .top_value    = 20000,
        .load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
        .step_mode    = NRF_PWM_STEP_AUTO
    };
    // Init PWM without error handler
    APP_ERROR_CHECK(nrf_drv_pwm_init(&m_pwm1, &config0, NULL));

    
    
    nrf_drv_pwm_simple_playback(&m_pwm1, &seq1, 1,NRF_DRV_PWM_FLAG_LOOP);
                
}
*/

static void pwm_init(void)
{
    check = NRF_SUCCESS; 
    nrf_drv_pwm_config_t const config0 =
    {
        .output_pins =
        {
            OUTPUT_PIN, // channel 0
            OUTPUT_PIN2,             // channel 1
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 2
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 3
        },
        .irq_priority = APP_IRQ_PRIORITY_LOWEST,
        .base_clock   = NRF_PWM_CLK_1MHz,
        .count_mode   = NRF_PWM_MODE_UP,
        .top_value    = 20000,
        .load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
        .step_mode    = NRF_PWM_STEP_AUTO
    };
    // Init PWM without error handler
     APP_ERROR_CHECK(nrf_drv_pwm_init(&m_pwm0, &config0, irq_handler));

    
    pwm_task_addr=nrf_drv_pwm_simple_playback(&m_pwm0, &seq, 1,NRF_DRV_PWM_FLAG_LOOP | NRF_DRV_PWM_FLAG_START_VIA_TASK);
                
}

/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    if (NRF_LOG_PROCESS() == false)
    {
        nrf_pwr_mgmt_run();
    }
}



int main(void)
{

  // Start accurate HFCLK (XOSC)
    NRF_CLOCK->TASKS_HFCLKSTART = 1;
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0) ;
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;

    uint32_t err_code;
    uint8_t count;
    err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);
    NRF_LOG_DEFAULT_BACKENDS_INIT();


    bsp_board_init(BSP_INIT_LEDS);
    NRF_LOG_INFO("LED started.");

    gpio_init();

    NRF_LOG_INFO("gpio started.");

    timers_init();

    NRF_LOG_INFO("timer  started.");

    //pwm1_init();

    pwm_init();
    
    NRF_LOG_INFO("pwm started.");
    
    ppi_init();
    NRF_LOG_INFO("PPI example started.");
    
    
    
    while (1)
    {
     
      

    }
}
